def parse_name(long_str):
    if long_str.count('.') and long_str.count('['):
        i = min(long_str.index('.'), long_str.index('['))
    elif long_str.count('.'):
        i = long_str.index('.')
    elif long_str.count(']'):
        i = long_str.index(']')
    else:
        i = 0
    key = long_str[:i] if i else long_str

    return key


def get_output(loc, glob, bins, long_str, color):
    # from .name_parser import parse_name

    is_undefined = False
    key = parse_name(long_str)
    _sep0 = ' '

    if key in loc:
        try:
            _val = eval(long_str, glob, loc)

            _src = 'L'
            _src = _src + ('g' if key in glob else '.')
            _src = _src + ('b' if key in bins else '.')
            _type = type(_val).__name__
            _sep1 = ':'
            _sep2 = '  =>  '
        except Exception:
            is_undefined = True

    elif key in glob:
        try:
            _val = eval(long_str, glob, loc)
            _src = '.G'
            _src = _src + ('b' if key in bins else '.')
            _type = type(_val).__name__
            _sep1 = ':'
            _sep2 = '  =>  '
        except Exception:
            is_undefined = True

    elif key in bins:
        try:
            _val = eval(long_str, bins)
            _src = '..B'
            _type = (
                type(_val).__name__
                if type(_val).__name__ != 'builtin_function_or_method'
                else 'function'
            )
            _sep1 = ':'
            _sep2 = '  =>  '
        except Exception:
            is_undefined = True

    else:
        is_undefined = True

    if is_undefined:
        _src = '...'
        _val = ''
        _type = ''
        _sep1 = ''
        _sep2 = ''

    if _type == 'function' and _val.__doc__:
        _desc = (
            f'{_val.__doc__}'[:40] + '...'
            if len(_val.__doc__) > 43
            else _val.__doc__
        )
    elif _type == 'function':
        _desc = 'No docstring'
    else:
        _desc = _val

    return (
        f'\u001b[36m{_src}\u001b[35m{_sep0}\u001b[0m{long_str}\u001b[35m{_sep1}\u001b[31m{_type}\u001b[35m{_sep2}\u001b[33m{_desc}\u001b[0m'
        if color
        else f'{_src}{_sep0}{long_str}{_sep1}{_type}{_sep2}{_desc}'
    )



from nezu.nezu import Nezu

def test_instances():
    n1 = Nezu('n1')
    n1(3)
    n2 = Nezu('n2')
    n2(6,1)
    assert dict(n1) == {'id': 'n1', 'seek': 3, 'color': False}
    assert dict(n2) == {'id': 'n2', 'seek': 6, 'color': True}


from nezu.nezu import real_nezu
from os import environ as env


def test_no_config():
    env['NEZU_SEEK'] = '0'
    env['NEZU_COLOR'] = '0'
    nezu = real_nezu()
    assert dict(nezu) == {'id': 'nezu', 'seek': 0, 'color': False}


def test_hard_config_seek():
    nezu = real_nezu()
    nezu(seek=1)
    assert dict(nezu) == {'id': 'nezu', 'seek': 1, 'color': False}


def test_hard_config_color():
    nezu = real_nezu()
    nezu(color=True)
    assert dict(nezu) == {'id': 'nezu', 'seek': 0, 'color': True}


def test_json_config_seek():
    content = '{"nezu":{"seek":1}}'
    with open('nezu.json', 'w') as f:
        print(content, file=f)
    nezu = real_nezu()
    nezu.json()
    assert dict(nezu) == {'id': 'nezu', 'seek': 1, 'color': False}


def test_json_config_color():
    content = '{"nezu":{"color":true}}'
    with open('nezu.json', 'w') as f:
        print(content, file=f)
    nezu = real_nezu()
    nezu.json()
    assert dict(nezu) == {'id': 'nezu', 'seek': 0, 'color': True}



from os import getenv


def os_bool(id: str, param: str):
    val = getenv(f'{id.upper()}_{param.upper()}')
    val = val.casefold() if val != None else 'False'
    val = (
        True
        if val in ('true', 't') or val.isdecimal() and int(val) != 0
        else False
    )
    return val


def os_int(id: str, param: str):
    val = getenv(f'{id.upper()}_{param.upper()}')
    val = int(val) if val != None else 0
    return val



from nezu.nezu import real_nezu

def test_hide(capsys):
    nezu = real_nezu()
    nezu(2)
    lis = [
        '--0--',
        '--1--',
        '--2--',
        '',
        '',
    ]
    for i in range(5):
        nezu.say(f'--{i}--', hide=i)
        said = capsys.readouterr().out
        assert said[8:-1] == lis[i]

def test_color(capsys):
    nezu = real_nezu()
    nezu(1, True)
    biggus = 'dickus'
    nezu.say(biggus)
    said = capsys.readouterr().out
    expected = f'\033[92mdickus\033[0m'
    assert said[8:-1] == expected


"""Very fragile test code. Do not attempt to reformat."""
from nezu.nezu import real_nezu

nezu = real_nezu()
nezu(1)


def test_line_checker(capsys):
    nezu.dbg()
    said = capsys.readouterr().out
    assert said.strip() == '@9'

    nezu.dbg()
    said = capsys.readouterr().out
    assert said.strip() == '@13'



"""Elegant debuging tool"""
if __name__ == '__main__':
    print('Nezu is elegant debuging module')
else:
    from inspect import currentframe
    from typing import Any, Self

    class Nezu:
        """Nezu object"""

        def __init__(self, id: str):
            from nezu._os_env import os_bool, os_int

            self._id = id
            self.seek = os_int(id, 'SEEK')
            self.color = os_bool(id, 'COLOR')
            self._locked = os_bool(id, 'LOCK')

            self.hard = self.__call__

        def __call__(
            self, seek: int = 0, color: bool = False, lock: bool = False
        ):
            """
            Hard coded configuration.

            Parameters
            ------------
                seek: int = 0
                    How deep to seek for hidden calls of `dbg` function?
                    dbg function is called with `hide` parameter (0 by default).
                    If `seek => hide` message will be displayed.
                color: bool = False
                    Determine if output is colored.

            """
            if not self._locked:
                self.seek = seek
                self.color = color
                self._locked = lock
            else:
                raise RuntimeError(
                    f'This Nezu object #{self._id} is locked and cannot be changed.'
                )

        def __iter__(self):
            return iter(
                [
                    ('id', self._id),
                    ('seek', self.seek),
                    ('color', self.color),
                ]
            )

        def json(self, path: str = 'nezu.json', **kwargs):
            """
            Initialaze nezu via json file
            """
            if not self._locked:
                import json

                with open(path, 'r', **kwargs) as file:
                    all_data = json.load(file)
                nezu_data = all_data.get(self._id, {})
                self.seek = nezu_data.get('seek', 0)
                self.color = nezu_data.get('color', False)

            else:
                raise RuntimeError(
                    f'This Nezu object #{self._id} is locked and cannot be changed.'
                )

        def say(self, var, hide: int = 1):
            if self.seek >= hide:
                FRAME = currentframe().f_back
                LINE = FRAME.f_lineno
                prfx = f'@{LINE}'.rjust(7) + ' '
                # palete = {
                #     'default': '\033[0m',
                #     'red': '\033[91m',
                #     'green': '\033[92m',
                #     'yellow': '\033[93m',
                #     'blue': '\033[94m',
                #     'purple': '\033[95m',
                #     'cyan': '\033[96m',
                # }
                print(
                    f'{prfx}\033[92m{var}\033[0m'
                    if self.color
                    else f'{prfx}{var}'
                )

        def dbg(self, *keys: str, note: str = None, hide: int = 1) -> None:
            """
            Parameters
            ------------

            - `*keys:str`

                Names of varables to inspect

            - `note:str=None`

                Optional comment. Ignored if equal to None.

            - `hide:int=1`

                How deep do you want to hide this message.
                If `hide <= 0`, this message will be displayed by default.


            Description
            ------------
            Function `dbg` displays following data in that order for each inspected varable:

            - number of line it was called at

            - scope of inspected variable

            - name of inspected variable

            - type of inspected variable

            - value of inspected variable
            """

            if self.seek >= hide:
                from nezu._parse import get_output

                FRAME = currentframe().f_back
                LINE = FRAME.f_lineno
                LOCAL = FRAME.f_locals
                GLOBAL = FRAME.f_globals
                BINS = FRAME.f_builtins

                if len(keys) < 2:
                    prfx = f'@{LINE}'.rjust(7) + ' '
                    desc = '  |  '.join(
                        [
                            get_output(LOCAL, GLOBAL, BINS, key, self.color)
                            for key in keys
                        ]
                    )
                    sufx = f'  |  << {note} >>' if note != None else ''
                else:
                    tab = f'\n{" "*8}'
                    prfx = f'@{LINE}'.rjust(7) + ' ' + '-' * 70 + tab
                    desc = tab.join(
                        [
                            get_output(LOCAL, GLOBAL, BINS, key, self.color)
                            for key in keys
                        ]
                    )
                    sufx = f'{tab}<< {note} >>' if note != None else ''
                    sufx += f'{tab}{"-"*70}'

                print(f'{prfx}{desc}{sufx}')

    def real_nezu():
        """
        Creates default instance of Nezu.
        # DO NOT USE THIS FUNCTION IN YOUR PROJECT!
        ---
        This function is only to be used in internal tests, so each test has unique Nezu instance identical to deafult Nezu instance.

        If you want multiple nezu instances, create them using constructor function `Nezu(id='my_nezu')`
        """
        defaults = {'id': 'nezu'}
        return Nezu(**defaults)

    ず = nezu = real_nezu()
    虫 = dbg = nezu.dbg
    話 = say = nezu.say



from nezu.nezu import real_nezu


def test_hide(capsys):
    nezu = real_nezu()
    nezu(2)
    lis = [
        'L.. i:int  =>  0',
        'L.. i:int  =>  1',
        'L.. i:int  =>  2',
        '',
        '',
    ]
    for i in range(5):
        nezu.dbg('i', hide=i)
        said = capsys.readouterr().out
        assert said[8:-1] == lis[i]


def test_color(capsys):
    nezu = real_nezu()
    nezu(1, True)
    biggus = 'dickus'
    nezu.dbg('biggus')
    said = capsys.readouterr().out
    expected = f'\u001b[36mL..\u001b[35m \u001b[0mbiggus\u001b[35m:\u001b[31mstr\u001b[35m  =>  \u001b[33mdickus\u001b[0m'
    assert said[8:-1] == expected


# def test_list(capsys):
#     nezu = real_nezu()
#     nezu(1)
#     biggus = ['dickus']
#     nezu.dbg('biggus[0]')
#     said = capsys.readouterr().out
#     expected = f'l biggus:str  =>  dickus'
#     assert said[8:-1] == expected


def test_obj(capsys):
    nezu = real_nezu()
    nezu(1)

    class Biggus:
        txt: str = 'dickus'

    biggus = Biggus()
    nezu.dbg('biggus.txt')
    said = capsys.readouterr().out
    expected = f'L.. biggus.txt:str  =>  dickus'
    assert said[8:-1] == expected


def test_nesterd_obj(capsys):
    nezu = real_nezu()
    nezu(1)

    class Egg:
        txt: str = 'bacon'

    class Ham:
        child = Egg()

    class Spam:
        child = Ham()

    spam = Spam()
    nezu.dbg('spam.child.child.txt')
    said = capsys.readouterr().out
    expected = f'L.. spam.child.child.txt:str  =>  bacon'
    assert said[8:-1] == expected


def test_scope_b(capsys):
    nezu = real_nezu()
    nezu(1)
    nezu.dbg('print')
    said = capsys.readouterr().out
    assert said[8:11] == f'..B'


def test_scope_l(capsys):
    nezu = real_nezu()
    nezu(1)
    ham = 'egg'
    nezu.dbg('ham')
    said = capsys.readouterr().out
    assert said[8:11] == f'L..'


global_biggus = 'dickus'


def test_scope_g(capsys):
    nezu = real_nezu()
    nezu(1)
    nezu.dbg('global_biggus')
    said = capsys.readouterr().out
    assert said[8:11] == f'.G.'


def test_shadowing_L_b(capsys):
    nezu = real_nezu()
    nezu(1)
    print = 'egg'
    nezu.dbg('print')
    said = capsys.readouterr().out
    assert said[8:11] == f'L.b'


global_nortius = 'maximus'


def test_shadowing_Lg(capsys):
    nezu = real_nezu()
    nezu(1)
    global_nortius = 'minimus'
    nezu.dbg('global_nortius')
    said = capsys.readouterr().out
    assert said[8:11] == f'Lg.'


default_breakpoint = breakpoint
breakpoint = 69


def test_shadowing_Gb(capsys):
    nezu = real_nezu()
    nezu(1)
    nezu.dbg('breakpoint')
    said = capsys.readouterr().out
    assert said[8:11] == f'.Gb'


def test_shadowing_Lgb(capsys):
    nezu = real_nezu()
    nezu(1)
    breakpoint = 420
    nezu.dbg('breakpoint')
    said = capsys.readouterr().out
    assert said[8:11] == f'Lgb'


breakpoint = default_breakpoint


def test_multiline(capsys):
    nezu = real_nezu()
    nezu(1)
    biggus = 'BIGGUS'
    dickus = 'DICKUS'
    nezu.dbg('biggus', 'dickus')
    said = capsys.readouterr().out
    a, b, c, d, _ = said.split('\n')
    assert a[8:] == '-' * 70
    assert b[8:] == f'L.. biggus:str  =>  BIGGUS'
    assert c[8:] == f'L.. dickus:str  =>  DICKUS'
    assert d[8:] == '-' * 70
    # assert said == ''



# file.py
from ..nezu import nezu, dbg

nezu(1)

egg = 3
ham = int()
spam = {'spam': 'bacon'}

dbg('egg')          # works on simple variables
dbg('ham.real')     # works on attributes
dbg('print')        # works on functions and build-ins
dbg('spam["spam"]')   # DOES NOT work on keys and indexes yet



from ..nezu import dbg

x = 13
dbg('x')
input = 12


def yolo():
    def yoyo():
        pass

    dbg('x')
    dbg('xx', 'x')
    dbg('globals')
    dbg('yolo', hide=3, note='hidden')
    dbg('yoyo')
    dbg('xx')
    dbg('input')


yolo()


class Dog:
    @staticmethod
    def sit(self):
        """wolololo"""
        pass

    def bark(self):
        dbg('self.sits')
        dbg('self.sit')
        # print(locals())


fifi = Dog()

fifi.bark()

# dbg({'x':x})
